@@ -15,9 +15,6 @@ import univention.admin.config
 import univention.admin.filter
 import univention.admin.handlers
 import univention.admin.handlers.dns.forward_zone
-import univention.admin.handlers.dns.reverse_zone
-import univention.admin.handlers.groups.group
-import univention.admin.handlers.networks.network
 import univention.admin.localization
 import univention.admin.modules
 import univention.admin.password
