@@ -7,7 +7,6 @@
 from __future__ import annotations
 
 import re
-from logging import getLogger
 from typing import Any
 
 import ldap
@@ -22,8 +21,6 @@ from univention.admin import configRegistry
 from univention.admin.layout import Group, Tab
 
 
-log = getLogger('ADMIN')
-
 translation = univention.admin.localization.translation('univention.admin.handlers.nagios')
 _ = translation.translate
 
@@ -131,7 +128,7 @@ class object(univention.admin.handlers.simpleLdap):
                 # find correct dNSZone entry
                 res = self.lo.authz_connection.search(filter=filter_format('(&(objectClass=dNSZone)(zoneName=%s)(relativeDomainName=%s)(aRecord=*))', (zoneName, relDomainName)))
                 if not res:
-                    log.debug('service.py: open: could not find dNSZone of %s', host)
+                    self.log.debug('open: could not find dNSZone', host=host)
                 else:
                     # found dNSZone
                     filter = '(&(objectClass=univentionHost)'
